import Roact from "@rbxts/roact"
import { useEffect } from "@rbxts/roact-hooked"
import { useWorldContext } from "../contexts/worldContext"
import { ContextActionService, HttpService } from "@rbxts/services"
import Padding from "ReplicatedStorage/ui/components/padding"
import Acrylic from "ReplicatedStorage/ui/components/acrylic"
import { Spring } from "@rbxts/flipper"
import { useGroupMotor } from "@rbxts/roact-hooked-plus"

interface SlotProps extends Roact.JsxInstanceProperties<Frame> {
    index: number
    keycode: Enum.KeyCode
    tool: Tool

    Event?: Roact.JsxInstanceEvents<Frame>
	Change?: Roact.JsxInstanceChangeEvents<Frame>
}

const SLOT_DEFAULT = [new Spring(.7, { frequency: 6 }), new Spring(6, { frequency: 6 })]
const SLOT_ACTIVE = [new Spring(.5, { frequency: 6 }), new Spring(5, { frequency: 6 })]

function slot(props: SlotProps): Roact.Element {
    const { index, keycode, tool } = props

    const spreadableProps = { ...props } as Partial<SlotProps>
    delete spreadableProps.index
    delete spreadableProps.keycode
    delete spreadableProps.tool

    const [slotBgTransparencyAndSlotRatio, setSlotGoal] = useGroupMotor([.7, 6])
    const slotBgTransparency = slotBgTransparencyAndSlotRatio.map((t) => t[0])
    const slotRatio = slotBgTransparencyAndSlotRatio.map((t) => t[1])

    const { clientState } = useWorldContext()

    const handleActivated = (): void => {
        if (tool.Parent !== clientState.character) {
            clientState.character.Humanoid.EquipTool(tool)
            setSlotGoal(SLOT_ACTIVE)
        } else {
            clientState.character.Humanoid.UnequipTools()
            setSlotGoal(SLOT_DEFAULT)
        }
    }

    // TODO: maybe opt this into our system for inputs?
    useEffect(() => {
        const guid = HttpService.GenerateGUID(false)
        ContextActionService.BindAction(
            guid,
            (_, userInputState) => {
                if (Enum.UserInputState.Begin === userInputState) handleActivated()
            },
            false,
            keycode
        )

        tool.Unequipped.Connect(() => {
            setSlotGoal(SLOT_DEFAULT)
        })
    })

    return (
        <frame
            BackgroundColor3={Color3.fromHex("#11111b")}
            BackgroundTransparency={slotBgTransparency}
            Size={new UDim2(1, 0, 1, 0)}
        >
            <Acrylic
                radius={2}
                distance={0.001}
            />
            <uiaspectratioconstraint
                AspectRatio={slotRatio}
            />
            <uicorner
                CornerRadius={new UDim(0, 2)}
            />
            <frame BackgroundTransparency={1} Size={new UDim2(1, 0, 1, 0)}>
                <Padding
                    paddingY={new UDim(.3, 0)}
                />
                <textlabel
                    TextColor3={Color3.fromHex("#cdd6f4")}
                    BackgroundTransparency={1}
                    Size={new UDim2(.2, 0, 1, 0)}
                    Font={Enum.Font.GothamBold}
                    Text={tostring(index)}
                    TextScaled
                />
                <textlabel
                    TextColor3={Color3.fromHex("#cdd6f4")}
                    BackgroundTransparency={1}
                    Size={new UDim2(.8, 0, 1, 0)}
                    Position={new UDim2(.2, 0, 0, 0)}
                    Font={Enum.Font.Gotham}
                    Text={tool.Name}
                    TextXAlignment={Enum.TextXAlignment.Left}
                    TextScaled
                />
            </frame>
        </frame>
    )
}

export default slot